/**********************************************************************
* procedure : Final Semester Project
*
*      file : ether.c
*
*       			(C) 1996, Brendan D. Donahe and Jerome C. Parks,
*							all rights reserved
*
*   project : snoop.exe
*
*   purpose : Allocates memory for Ether filtering and allocates
*							linked list of buffers for handling network bursts
*
*     usage : void eth_clear_table();
*							void build_buffers(PKTBUF far **head_buffer);
*
* exit code : None
*
*   authors : Brendan D. Donahe and
*							Jerome C. Parks,
*							New Mexico State University, EE 464
*
*  language : Borland International C/C++, version 2.0
*							plus corresponding assembler and linker
*
*   history : December 13, 1996,  Verion 1.0
*
***********************************************************************/

#include <stdio.h>
#include <alloc.h>
#include <stdlib.h>

#include "pktdrvr.h"
#include "ether.h"
#include "snoop.h"

void eth_clear_table() {
	int i;
	int j;

	eth_src_tbl   = (ETH_ADDR_TBL far *)farmalloc(ETHER_TABLE_LEN*
									sizeof(ETH_ADDR_TBL));

	if(!eth_src_tbl) {
		fprintf(stderr,"Could not allocate memory for eth_src_tbl in clear_table.\n");
		exit(1);
	}

	eth_dst_tbl   = (ETH_ADDR_TBL far *)farmalloc(ETHER_TABLE_LEN*
									sizeof(ETH_ADDR_TBL));

	if(!eth_dst_tbl) {
		fprintf(stderr,"Could not allocate memory for eth_dst_tbl in clear_table.\n");
		exit(1);
	}

	eth_alias_tbl = (ETH_ALIAS_TBL far *)farmalloc(ETHER_TABLE_LEN*
									sizeof(ETH_ALIAS_TBL));

	if(!eth_alias_tbl) {
		fprintf(stderr,"Could not allocate memory for eth_alias_tbl in clear_table.\n");
		exit(1);
	}

	eth_type_tbl  = (ETH_TYPE_TBL far *)farmalloc(ETHER_TABLE_LEN*
									sizeof(ETH_TYPE_TBL));

	if(!eth_type_tbl) {
		fprintf(stderr,"Could not allocate memory for eth_type_tbl in clear_table.\n");
		exit(1);
	}


	for (i=0; i < ETHER_TABLE_LEN; i++) {
		for (j=0; j < ETHER_ADDR_LEN; j++)
			eth_src_tbl[i].addr[j]  = eth_dst_tbl[i].addr[j]
															= eth_alias_tbl[i].addr[j] = 0;

		eth_alias_tbl[i].alias[0] = '\0';

		eth_src_tbl[i].logicop    = eth_dst_tbl[i].logicop
															= ETHER_OR | ETHER_NOT_NOT;

		eth_src_tbl[i].index      = eth_dst_tbl[i].index    = 0;

		eth_type_tbl[i].type		  = 0;
	}

	return;
}


void build_buffers(PKTBUF far **head_buffer) {
	PKTBUF far *temp_buf;
	int i;

	*head_buffer = (PKTBUF far *)farmalloc(sizeof(PKTBUF));

	if(!*head_buffer) {
		fprintf(stderr, "Could not allocate memory for *head_buffer in build_buffers.\n");
		exit(1);
	}

	temp_buf = *head_buffer;
	temp_buf->pktsize_flag = 0;
	temp_buf->buffer = (char far *)farmalloc(sizeof(char)*ETHER_BUFSIZ);
	if(!temp_buf->buffer) {
		fprintf(stderr, "Could not allocate memory for *head_buffer->buffer in build_buffers.\n");
		exit(1);
	}
	temp_buf->next = (PKTBUF far *)farmalloc(sizeof(PKTBUF));
	if(!temp_buf->next) {
		fprintf(stderr, "Could not allocate memory for *head_buffer->next in build_buffers.\n");
		exit(1);
	}

	for(i=0;i<N_BUFS-1;i++) {        			/* set up linked list of N_BUFS */
		temp_buf = temp_buf->next;
		temp_buf->pktsize_flag = 0;					/* buffers */
		temp_buf->buffer = (char far *)farmalloc(sizeof(char)*ETHER_BUFSIZ);
		if(!temp_buf->buffer) {
			fprintf(stderr, "Could not allocate memory for *head_buffer->buffer in build_buffers.\n");
			exit(1);
		}
		temp_buf->next = (PKTBUF far *)farmalloc(sizeof(PKTBUF));
		if(!temp_buf->next) {
			fprintf(stderr, "Could not allocate memory for temp_buf->next (%d) in build_buffers.\n", i);
			exit(1);
		}
	}

	temp_buf->next = *head_buffer;					/* make linked list circular */

	return;
}


