/**********************************************************************
* procedure : Final Semester Project
*
*      file : ethfltr.c
*
*       			(C) 1996, Brendan D. Donahe and Jerome C. Parks,
*							all rights reserved
*
*   project : snoop.exe
*
*   purpose : Checks for matches between addresses, types, and filters
*
*     usage : int eth_addr_filter(ETH_HEADER far *buf);
*							int eth_type_filter(ETH_HEADER far *buf);
*
* exit code : 1 = Match, 0 = No Match
*
*   authors : Brendan D. Donahe and
*							Jerome C. Parks,
*							New Mexico State University, EE 464
*
*  language : Borland International C/C++, version 2.0
*							plus corresponding assembler and linker
*
*   history : December 13, 1996,  Verion 1.0
*
***********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "pktdrvr.h"
#include "ether.h"
#include "snoop.h"

int cmp_eth_addr(char far *addr1, char far *addr2) {
	int i;
	int flag=1;

	for(i=0; (i < ETHER_ADDR_LEN) && flag; i++)
		if(addr1[i]!=addr2[i]) flag=0;

	return(flag);
}

int cmp_eth_type(unsigned int type1, unsigned int type2) {
	return(type1==type2);
}

int eth_addr_filter(ETH_HEADER far *buf) {
	int index;
	int	ethaddr_flag=0;

	for(index=0; (index < eth_src_index) && !ethaddr_flag;
																						index++) {  // find source match
		ethaddr_flag = cmp_eth_addr(eth_src_tbl[index].addr,buf->src);
		if (ethaddr_flag)																		// check for dest AND match
			if ((eth_src_tbl[index].logicop & ETHER_ANDOR_MASK) == ETHER_AND)
				ethaddr_flag =
					cmp_eth_addr(eth_dst_tbl[eth_src_tbl[index].index].addr,buf->dst);
	}

	for(index=0; (index < eth_dst_index) && !ethaddr_flag; index++) {
		ethaddr_flag = cmp_eth_addr(eth_dst_tbl[index].addr,buf->dst);
			if (ethaddr_flag)
			if ((eth_dst_tbl[index].logicop & ETHER_ANDOR_MASK) == ETHER_AND)
				ethaddr_flag =
					cmp_eth_addr(eth_src_tbl[eth_dst_tbl[index].index].addr,buf->src);
	}

	return(ethaddr_flag);
}


int eth_type_filter(ETH_HEADER far *buf) {
	int index;
	int ethtype_flag;

	for(index=0; (index < eth_type_index) && !ethtype_flag; index++)
	ethtype_flag = cmp_eth_type(eth_type_tbl[index].type,buf->type);

	if(eth_type_index==0) ethtype_flag=1;

	return(ethtype_flag);
}


int eth_alias(char far *addr, char far **returnstr) {
	int 						 index;
	int			  			 ethaddr_flag=0;

	*returnstr = NULL;

	for(index=0; (index < eth_alias_index) && !ethaddr_flag; index ++) {
		ethaddr_flag = cmp_eth_addr(eth_alias_tbl[index].addr,addr);
		if (ethaddr_flag)
			if (eth_alias_tbl[index].alias[0])
				*returnstr = (char far *)eth_alias_tbl[index].alias;
			else ethaddr_flag = 0;
	}


	return(ethaddr_flag);
}



