/**********************************************************************
* procedure : Final Semester Project
*
*      file : ether.h
*
*       			(C) 1996, Brendan D. Donahe and Jerome C. Parks,
*							all rights reserved
*
*   project : snoop.exe
*
*   purpose : To set up defines, types, and structs for use with ether
*
*   authors : Brendan D. Donahe and
*							Jerome C. Parks,
*							New Mexico State University, EE 464
*
*  language : Borland International C/C++, version 2.0
*							plus corresponding assembler and linker
*
*   history : December 13, 1996,  Verion 1.0
*
***********************************************************************/

/* ethernet types */
#define	ETHER_NS_IDP				0x0600	// Xerox NS IDP
#define	ETHER_IP						0x0800  // IP
#define	ETHER_X75						0x0801	// X.75 internet
#define	ETHER_NBS						0x0802	// NBS internet
#define	ETHER_ECMA					0x0803	// ECMA internet
#define	ETHER_CHAOS					0x0804	// ChaosNet
#define	ETHER_X25						0x0805	// X.25, level 3
#define	ETHER_ARP						0x0806	// ARP
#define	ETHER_XNS						0x0807	// XNS compatibility
#define	ETHER_DEC						0x6003  // DECnet, phase IV
#define	ETHER_VLN						0x8003  // Cronus VLN
#define	ETHER_HP						0x8005  // HP Probe
#define	ETHER_NESTAR				0x8006  // Nestar
#define	ETHER_EXCELAN				0x8010  // Excelan
#define	ETHER_RARP					0x8035	// reverse ARP
#define	ETHER_LAN						0x8038  // DEC LANbridge
#define	ETHER_APPLE					0x809b  // Appletalk
#define	ETHER_LOOP					0x9000  // loopback

#define ETHER_BUFSIZ				1536		// size of ether buffer
#define	ETHER_ADDR_LEN			6       // size of an ether address
#define ETHER_ALIAS_LEN			81			// ethernet alias buffer size

#define ETHER_ADDR_1_EO 		21			// end of ethernet address 1 length +1
#define ETHER_ADDR_2_EO			39			// end of ehternet address 2 length +1

#define ETHER_TABLE_LEN 		128			// arbitrary ether table length

#define	ETHER_ANDOR_MASK		0x1			// mask off everything but first bit (lsb)
#define	ETHER_OR						0x0			// OR value
#define ETHER_AND						0x1			// AND value (use index to find pair)

#define ETHER_NOT_MASK			0x2			// mask off everything but second bit
#define ETHER_NOT_NOT				0x0			// NOT NOT value
#define ETHER_NOT						0x2			// NOT value

typedef struct {										// ethernet filtering addr table struct
	char					addr[ETHER_ADDR_LEN];
	char					logicop;
	unsigned int  index;
} ETH_ADDR_TBL;

typedef struct {										// ethernet filtering type table struct
	unsigned int	type;
}	ETH_TYPE_TBL;

typedef struct {										// ethernet displaying alias table struct
	char					addr[ETHER_ADDR_LEN];
	char					alias[ETHER_ALIAS_LEN];
} ETH_ALIAS_TBL;

typedef struct {										// ethernet header struct
	unsigned char dst[ETHER_ADDR_LEN];
	unsigned char src[ETHER_ADDR_LEN];
	unsigned int	type;
} ETH_HEADER;


/* linked list buffer structure */
#define N_BUFS	64							// was 8, ok for 486s and pentium


typedef struct PKTBUF_STRUCT {
	volatile unsigned int     pktsize_flag;
	char far								 *buffer;
	struct PKTBUF_STRUCT far *next;
} PKTBUF;

/* filter.c */
int eth_filterdst(ETH_HEADER far *buf);

int eth_filtersrc(ETH_HEADER far *buf);

int eth_addr_filter(ETH_HEADER far *buf);

int	eth_type_filter(ETH_HEADER far *buf);

int eth_alias(char far *addr, char far **returnstr);

/* ether.c */
void build_buffers(PKTBUF far **head_buffer);

void eth_clear_table(void);

/* ethhead.c */
void eth_header(ETH_HEADER far *MAC_header, unsigned int length);

#ifndef	 SNOOP_MAIN									// global vars & pointers
	extern ETH_ADDR_TBL  far *eth_src_tbl; 				  // src ether address
	extern ETH_ADDR_TBL  far *eth_dst_tbl; 					// dest ether address
	extern ETH_TYPE_TBL  far *eth_type_tbl;					// ethernet type
	extern ETH_ALIAS_TBL far *eth_alias_tbl;        // ethernet alias
	extern int eth_src_index;							// pointers to first unused
	extern int eth_dst_index;							//   entry in the tables
	extern int eth_type_index;
	extern int eth_alias_index;
#endif

