/**********************************************************************
* procedure : Final Semester Project
*
*      file : ip.h
*
*       			(C) 1996, Brendan D. Donahe and Jerome C. Parks,
*							all rights reserved
*
*   project : snoop.exe
*
*   purpose : Defines, prototypes, and structs for use with IP
*
*   authors : Brendan D. Donahe and
*							Jerome C. Parks,
*							New Mexico State University, EE 464
*
*  language : Borland International C/C++, version 2.0
*							plus corresponding assembler and linker
*
*   history : December 13, 1996,  Verion 1.0
*
***********************************************************************/

/* IP protocol types */
#define	IP_ICMP					  1                    //  internet control msg
#define	IP_IGMP				    2                    //  internet group mangmt
#define	IP_GGP						3      	             //  gateway-2-gateway
#define	IP_ST					    5    	               //  stream
#define	IP_TCP						6      	             //  tranmission control
#define	IP_UCL					  7                    //  UCL
#define	IP_EGP					  8      	             //  exterior gateway
#define	IP_IGP				    9     	             //  any private interior
#define	IP_BBN						10                   //  BBN RCC monitoring
#define	IP_NVP				    11                   //  network voice
#define	IP_PUP						12                   //  PUP
#define	IP_ARGUS				  13                   //  ARGUS
#define	IP_EMCON					14                   //  EMCON
#define	IP_XNET				    15                   //  cross-net debugger
#define	IP_CHAOS			    16      	           //  chaos
#define	IP_UDP						17                   //  user datagram
#define	IP_MUX				    18                   //  multiplexing
#define	IP_DCN					  19                   //  DCN measurment
#define	IP_HMP					  20                   //  host monitoring
#define	IP_PRM					  21                   //  packet radio measurment
#define	IP_XNS					  22                   //  XEROX NS IDP
#define	IP_TRNK1			    23                   //  trunk 1
#define	IP_TRNK2			    24                   //  trunk 2
#define	IP_LEAF1			    25                   //  leaf 1
#define	IP_LEAF2			    26                   //  leaf 2
#define	IP_RDP					  27                   //  reliable data
#define	IP_IRTP				    28                   //  inet reliable transaction
#define	IP_TP4					  29                   //  ISO transport, class 4
#define	IP_NETBLT		      30                   //  bulk data transfer
#define	IP_MFE					  31                   //  MFE net services
#define	IP_INP					  32                   //  merit internodal
#define	IP_SEP					  33                   //  sequential exchange
#define	IP_IHOST			    61                   //  any host internal
#define	IP_CFTP				    62                   //  CFTP
#define	IP_LOCL				    63                   //  any locak network
#define	IP_EXPAK			    64                   //  SATNET & backroom EXPAK
#define	IP_MIT					  65                   //  MIT subnet support
#define	IP_RVD					  66                   //  MIT remote virtual disk
#define	IP_IPPC				    67                   //  inet pluribus packet core
#define	IP_DFS 				    68                   //  any distributed file sys
#define	IP_SAT					  69                   //  SATNET monitoring
#define	IP_IPCU				    71                   //  inet packet core utility
#define	IP_BRSAT			    76                   //  backroom SATNET monitor
#define	IP_WBMON			    78                   //  wideband monitoring
#define	IP_WBEXPAK	      79                   //  wideband EXPAK


#define IP_ADDR_LEN					4       // size of an internet address
#define IP_ALIAS_LEN				81			// ip alias buffer size

#define IP_TABLE_LEN				128			// arbitrary ip table length

#define IP_ANDOR_MASK				0x1			// mask off everything but first bit (lsb)
#define	IP_OR								0x0			// OR value
#define IP_AND							0x1			// AND value (use index to find pair)

#define IP_NOT_MASK					0x2			// mask off everything but second bit
#define IP_NOT_NOT					0x0			// NOT NOT value
#define IP_NOT							0x2			// NOT value

typedef struct {										// ip filtering addr table struct
	char					addr[IP_ADDR_LEN];
	char					logicop;
	unsigned int  index;
	char					class;							// subnet class
} IP_ADDR_TBL;

typedef struct {										// ip filtering type table struct
	unsigned int	type;
}	IP_TYPE_TBL;

typedef struct {										// ip displaying alias table struct
	char					addr[IP_ADDR_LEN];
	char					alias[IP_ALIAS_LEN];
} IP_ALIAS_TBL;

typedef struct {
	unsigned char ether_dst[ETHER_ADDR_LEN];
	unsigned char ether_src[ETHER_ADDR_LEN];
	unsigned int	ether_type;

	unsigned char vers_IHL;		// first 4 bits the version number
														// second 4 bits the length in 4 octet
														//   units in header (internet header length)
	unsigned char TOS;				// type of service (priority)
	unsigned int  length;	    // length of packet
	unsigned char identification[2];
	unsigned char	flags_offset[2];
	unsigned char TTL;				// time to live
	unsigned char type;				// TCP, UDP, ICMP?
	unsigned char checksum[2];
	unsigned char	src[IP_ADDR_LEN];
	unsigned char dst[IP_ADDR_LEN];
	unsigned char	options[3];
} IP_HEADER;

/* filter.c */
int ip_filterdst(IP_HEADER far *buf);

int ip_filtersrc(IP_HEADER far *buf);

int ip_addr_filter(IP_HEADER far *buf);

int ip_type_filter(IP_HEADER far *buf);

int ip_alias(char far *addr, char far **returnstr);

/* ip.c */
void ip_clear_table(void);

/* iphead.c */
void ip_header(IP_HEADER far *header);

#ifndef	 SNOOP_MAIN									// global vars & pointers
	extern IP_ADDR_TBL  far *ip_src_tbl; 				  // src ether address
	extern IP_ADDR_TBL  far *ip_dst_tbl; 					// dest ether address
	extern IP_TYPE_TBL  far *ip_type_tbl;					// ethernet type
	extern IP_ALIAS_TBL far *ip_alias_tbl;        // ethernet alias
	extern int ip_src_index;							// pointers to first unused
	extern int ip_dst_index;							//   entry in the tables
	extern int ip_type_index;
	extern int ip_alias_index;
#endif
