/**********************************************************************
* procedure : Final Semester Project
*
*      file : tcp.h
*
*       			(C) 1996, Brendan D. Donahe and Jerome C. Parks,
*							all rights reserved
*
*   project : snoop.exe
*
*   purpose : Defines, prototypes, and structs for use with TCP
*
*   authors : Brendan D. Donahe and
*							Jerome C. Parks,
*							New Mexico State University, EE 464
*
*  language : Borland International C/C++, version 2.0
*							plus corresponding assembler and linker
*
*   history : December 13, 1996,  Version 1.0
*
***********************************************************************/

/* TCP protocol types */
#define TCP_TCPMUX							1
#define TCP_ECHO								7
#define TCP_DISCARD							9
#define TCP_SYSTAT							11
#define TCP_DAYTIME							13
#define TCP_NETSTAT							15
#define TCP_QOTD								17
#define TCP_MSP									18
#define TCP_CHARGEN							19
#define TCP_FTP_DATA						20
#define TCP_FTP									21
#define TCP_SSH									22
#define TCP_TELNET							23
#define TCP_SMTP								25
#define TCP_TIME								37
#define TCP_NAMESERVER					42
#define TCP_WHOIS								43
#define TCP_RE_MAIL_CK					50
#define TCP_DOMAIN							53
#define TCP_MTP									57
#define TCP_BOOTPS							67
#define TCP_BOOTPC							68
#define TCP_GOPHER							70
#define TCP_RJE									77
#define TCP_FINGER							79
#define TCP_WWW									80
#define TCP_LINK								87
#define TCP_KERBEROS						88
#define TCP_SUPDUP							95
#define TCP_HOSTNAMES						101
#define TCP_ISO_TSAP						102
#define TCP_CSNET_NS						105
#define TCP_RTELNET							107
#define TCP_POP_2								109
#define TCP_POP_3								110
#define TCP_SUNRPC							111
#define TCP_AUTH								113
#define TCP_SFTP								115
#define TCP_UUCP_PATH						117
#define TCP_NNTP								119
#define TCP_NTP									123
#define TCP_NETBIOS_NS					137
#define TCP_NETBIOS_DGM					138
#define TCP_NETBIOS_SSN					139
#define TCP_IMAP2								143
#define TCP_CMIP_MAN						163
#define TCP_CMIP_AGENT					164
#define TCP_XDMCP								177
#define TCP_NEXTSTEP						178
#define TCP_BGP									179
#define TCP_PROSPERO						191
#define TCP_IRC									194
#define TCP_SMUX								199
#define TCP_AT_RTMP							201
#define TCP_AT_NBP							202
#define TCP_AT_ECHO							204
#define TCP_AT_ZIS							206
#define TCP_Z3950								210
#define TCP_IPX									213
#define TCP_IMAP3								220
#define TCP_ULISTSERV						372
#define TCP_EXEC								512
#define TCP_LOGIN								513
#define TCP_SHELL								514
#define TCP_PRINTER							515
#define TCP_TEMPO								526
#define TCP_COURIER							530
#define TCP_CONFERENCE					531
#define TCP_NETNEWS							532
#define TCP_UUCP								540
#define TCP_REMOTEFS						556
#define TCP_KLOGIN							543
#define TCP_KSHELL							544
#define TCP_KERBEROS_ADM				749
#define TCP_WEBSTER							765
#define TCP_INGRESLOCK					1524
#define TCP_PROSPERO_NP					1525
#define TCP_RFE									5002
#define TCP_BBS									7000
#define TCP_KERBEROS4						750
#define TCP_KERBEROS_MASTER			751
#define TCP_KRB_PROP						754
#define TCP_KRBUPDATE						760
#define TCP_KPASSWD							761
#define TCP_KPOP								1109
#define TCP_KNETD								2053
#define TCP_EKLOGIN							2105
#define TCP_SUPFILESRV					871
#define TCP_SUPFILEDBG					1127
#define TCP_RMTCFG							1236
#define TCP_XTEL								1313
#define TCP_CFINGER							2003
#define TCP_POSTGRES						4321

#define TCP_PORT_LEN				2       // size of a tcp port
#define TCP_ALIAS_LEN				81			// tcp alias buffer size

#define TCP_TABLE_LEN				128			// arbitrary tcp table length

#define TCP_ANDOR_MASK			0x1			// mask off everything but first bit (lsb)
#define	TCP_OR							0x0			// OR value
#define TCP_AND							0x1			// AND value (use index to find pair)

#define TCP_NOT_MASK				0x2			// mask off everything but second bit
#define TCP_NOT_NOT					0x0			// NOT NOT value
#define TCP_NOT							0x2			// NOT value

typedef struct {										// tcp filtering port table struct
	unsigned int  port;
	char					logicop;
	unsigned int  index;
} TCP_PORT_TBL;

typedef struct {										// tcp filtering type table struct
	unsigned int	type;
}	TCP_TYPE_TBL;

typedef struct {										// tcp displaying alias table struct
	unsigned int	port;
	char					alias[TCP_ALIAS_LEN];
} TCP_ALIAS_TBL;

typedef struct {
	unsigned int			src;                  // 2
	unsigned int			dst;                  // 2
	unsigned long int sequence;             // 4
	unsigned long int	acknowledge;          // 4
	unsigned char     offset_reserved;      // 1
	unsigned char			control;              // 1
	unsigned int			window;               // 2
	unsigned int			checksum;             // 2
	unsigned int			urgent_ptr;           // 2
	unsigned char			options[3];           // 3  23
} TCP_HEADER;

/* filter.c */
int tcp_filterdst(TCP_HEADER far *buf);

int tcp_filtersrc(TCP_HEADER far *buf);

int tcp_type_filter(TCP_HEADER far *buf);

/* tcp.c */
void tcp_clear_table(void);

/* tcphead.c */
void tcp_header(TCP_HEADER far *header);

char far *tcp_port_type(unsigned int portnum);

#ifndef	 SNOOP_MAIN									// global vars & pointers
	extern TCP_PORT_TBL  far *tcp_src_tbl;
	extern TCP_PORT_TBL  far *tcp_dst_tbl;
	extern TCP_TYPE_TBL  far *tcp_type_tbl;
	extern TCP_ALIAS_TBL far *tcp_alias_tbl;
	extern int tcp_src_index;
	extern int tcp_dst_index;
	extern int tcp_type_index;
	extern int tcp_alias_index;
#endif
