/**********************************************************************
* procedure : Final Semester Project
*
*      file : iphead.c
*
*       			(C) 1996, Brendan D. Donahe and Jerome C. Parks,
*							all rights reserved
*
*   project : snoop.exe
*
*   purpose : Displays IP header information to log file
*
*     usage : void ip_header(IP_HEADER far *header);
*
* exit code : None
*
*   authors : Brendan D. Donahe and
*							Jerome C. Parks,
*							New Mexico State University, EE 464
*
*  language : Borland International C/C++, version 2.0
*							plus corresponding assembler and linker
*
*   history : December 13, 1996,  Verion 1.0
*
***********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <dos.h>		// for gettime and getdate

#include "pktdrvr.h"
#include "ether.h"
#include "ip.h"
#include "snoop.h"

void ip_header(IP_HEADER far *header) {
	char far *srcalias;
	char far *dstalias;

	srcalias = NULL;
	dstalias = NULL;

	if (ip_alias((char far *)(header->src),&srcalias))
		fprintf(log_file,"  IP:    len %4u  src %-17Fs  dst ",header->length,
									srcalias);
	else fprintf(log_file,"  IP:    len %4u  src %3i.%3i.%3i.%3i    dst ",
								header->length,
								header->src[0],header->src[1],header->src[2],
								header->src[3]);

	if (ip_alias((char far *)(header->dst),&dstalias))
		fprintf(log_file,"%-17Fs  type ",dstalias);
	else fprintf(log_file,"%3i.%3i.%3i.%3i    type ",
							 header->dst[0],header->dst[1],header->dst[2],
							 header->dst[3]);

	switch(header->type) {
	case	IP_ICMP:	// internet control msg
		fprintf(log_file,"ICMP");
		break;

	case	IP_IGMP:	// internet group mangmt
		fprintf(log_file,"IGMP");
		break;

	case	IP_GGP:		// gateway-2-gateway
		fprintf(log_file,"GGP");
		break;

	case	IP_ST:		// stream
		fprintf(log_file,"ST");
		break;

	case	IP_TCP:		// tranmission control
		fprintf(log_file,"TCP");
		break;

	case	IP_UCL:		// UCL
		fprintf(log_file,"UCL");
		break;

	case	IP_EGP:		// exterior gateway
		fprintf(log_file,"EGP");
		break;

	case	IP_IGP:		// any private interior
		fprintf(log_file,"IGP");      // gateway
		break;

	case	IP_BBN:		// BBN RCC monitoring
		fprintf(log_file,"BBN-RCC-MON");
		break;

	case	IP_NVP:		// network voice
		fprintf(log_file,"NVP-II");
		break;

	case	IP_PUP:		// PUP
		fprintf(log_file,"PUP");
		break;

	case	IP_ARGUS:	// ARGUS
		fprintf(log_file,"ARGUS");
		break;

	case	IP_EMCON:	// EMCON
		fprintf(log_file,"EMCON");
		break;

	case	IP_XNET:	// cross-net debugger
		fprintf(log_file,"XNET");
		break;

	case	IP_CHAOS:	// chaos
		fprintf(log_file,"CHAOS");
		break;

	case	IP_UDP:		// user datagram
		fprintf(log_file,"UDP");
		break;

	case	IP_MUX:		// multiplexing
		fprintf(log_file,"MUX");
		break;

	case	IP_DCN:		// DCN measurment
		fprintf(log_file,"DCN-MEAS");
		break;

	case	IP_HMP:		// host monitoring
		fprintf(log_file,"HMP");
		break;

	case	IP_PRM:		// packet radio measurment
		fprintf(log_file,"PRM");
		break;

	case	IP_XNS:		// XEROX NS IDP
		fprintf(log_file,"XNS-IDP");
		break;

	case	IP_TRNK1:	// trunk 1
		fprintf(log_file,"TRUNK-1");
		break;

	case	IP_TRNK2:	// trunk 2
		fprintf(log_file,"TRUNK-2");
		break;

	case	IP_LEAF1:	// leaf 1
		fprintf(log_file,"LEAF-1");
		break;

	case	IP_LEAF2:	// leaf 2
		fprintf(log_file,"LEAF-2");
		break;

	case	IP_RDP:		// reliable data
		fprintf(log_file,"RDP");
		break;

	case	IP_IRTP:	// inet reliable transaction
		fprintf(log_file,"IRTP");
		break;

	case	IP_TP4:		// ISO transport, class 4
		fprintf(log_file,"ISO-TP4");
		break;

	case	IP_NETBLT:	// bulk data transfer
		fprintf(log_file,"NETBLT");
		break;

	case	IP_MFE:		// MFE net services
		fprintf(log_file,"MFE-NSP");
		break;

	case	IP_INP:		// merit internodal
		fprintf(log_file,"MERIT-INP");
		break;

	case	IP_SEP:		// sequential exchange
		fprintf(log_file,"SEP");
		break;

	case	IP_IHOST:	// any host internal
		fprintf(log_file,"IHost");	// protocol
		break;

	case	IP_CFTP:	// CFTP
		fprintf(log_file,"CFTP");
		break;

	case	IP_LOCL:	// any locak network
		fprintf(log_file,"Local");
		break;

	case	IP_EXPAK:	// SATNET & backroom EXPAK
		fprintf(log_file,"SAT-EXPAK");
		break;

	case	IP_MIT:		// MIT subnet support
		fprintf(log_file,"MIT-SUBNET");
		break;

	case	IP_RVD:		// MIT remote virtual disk
		fprintf(log_file,"RVD");
		break;

	case	IP_IPPC:	// inet pluribus packet core
		fprintf(log_file,"IPPC");
		break;

	case	IP_DFS:		// any distributed file sys
		fprintf(log_file,"DFS");
		break;

	case	IP_SAT:		// SATNET monitoring
		fprintf(log_file,"SAT-MON");
		break;

	case	IP_IPCU:	// inet packet core utility
		fprintf(log_file,"IPCU");
		break;

	case	IP_BRSAT:	// backroom SATNET monitor
		fprintf(log_file,"BR-SAT-MON");
		break;

	case	IP_WBMON:	// wideband monitoring
		fprintf(log_file,"WB-MON");
		break;

	case	IP_WBEXPAK:	// wideband EXPAK
		fprintf(log_file,"WB-EXPAK");
		break;

	default:
		fprintf(log_file,"unknown");
	}

	fprintf(log_file,"\n");

}