/**********************************************************************
* procedure : Final Semester Project
*
*      file : parseopt.c
*
*       			(C) 1996, Brendan D. Donahe and Jerome C. Parks,
*							all rights reserved
*
*   project : snoop.exe
*
*   purpose : To parse through all command line arguments and call the
*							appropriate subroutines
*
*   actions : 1. Parse through all command line argumens and script
*								 files until match (or no match) is found
*
*     usage : char *parse_ip_src_addr(char *addr, int *start);
*
* exit code : 1 implies error status, 0 imples successful
*
*   authors : Brendan D. Donahe and
*							Jerome C. Parks,
*							New Mexico State University, EE 464
*
*  language : Borland International C/C++, version 2.0
*							plus corresponding assembler and linker
*
*   history : December 13, 1996,  Verion 1.0
*
***********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>

#include "pktdrvr.h"
#include "ether.h"
#include "ip.h"
#include "tcp.h"
#include "snoop.h"

char *parse_ip_src_addr(char *addr, int *start) {
	int  i=0;
	int  j=0;
	int  k;
	char wildcard=0;

	j=*start;

	for(k=0; k<4; k++) {
		for(; addr[j] && addr[j]!=',' && addr[j]!='.' && addr[j]!='(' &&
				addr[j]!='*' &&	!isdigit(addr[j]); j++);
		for(i=j; addr[i] && addr[i]!=',' && addr[i]!='.' && addr[i]!='(' &&
				(isdigit(addr[i]) || addr[i]=='*');	i++);
		if(!wildcard) {
			if(addr[j] && addr[j]=='*' && k>0) {
				ip_src_tbl[ip_src_index].class = k+'a'-1;
				wildcard=1;
			}	else {
				if(addr[i] && addr[i]!=',' && addr[i]!='.' && k!=3)
					return("Missing parameter in IP source address (/is or /ib).");
				if(!addr[i] && k!=3) return("Premature ending of IP source address (/is or /ib).");
				if(j==i) return("Premature ending of IP source address (/is or /ib).");
				ip_src_tbl[ip_src_index].addr[k] = (char)atoi(&addr[j]);
			}
		}
		if(addr[i]!=',') j=i+1;
	}

	*start=i;

	return("");
}

char *parse_ip_dst_addr(char *addr, int *start) {
	int  i=0;
	int  j=0;
	int  k;
	char wildcard=0;

	j=*start;

	for(k=0; k<4; k++) {
		for(; addr[j] && addr[j]!='.' && addr[j]!='(' && addr[j]!='*' &&
				!isdigit(addr[j]); j++);
		for(i=j; addr[i] && addr[i]!='.' && addr[i]!='(' &&
				isdigit(addr[i]);	i++);
		if(!wildcard) {
			if(addr[j] && addr[j]=='*' && k>0) {
				ip_dst_tbl[ip_dst_index].class = k+'a'-1;
				wildcard=1;
			}	else {
				if(addr[i] && addr[i]!='.' && k!=3)
					return("Missing parameter in IP destination address (/id or /ib).");
				if(!addr[i] && k!=3) return("Premature ending of IP destination address (/id or /ib).");
				if(j==i) return("Premature ending of IP destination address (/id or /ib).");
				ip_dst_tbl[ip_dst_index].addr[k] = (char)atoi(&addr[j]);
			}
		}
		j=i+1;
	}

	*start=i;

	return("");
}

char *parse_ip_alias_addr(char *addr, int *start) {
	int  i=0;
	int  j=0;
	int  k;

	j=*start;

	for(k=0; k<4; k++) {
		for(; addr[j] && addr[j]!='.' && addr[j]!='(' && addr[j]!='*' &&
				!isdigit(addr[j]); j++);
		for(i=j; addr[i] && addr[i]!='.' && addr[i]!='(' &&
				isdigit(addr[i]);	i++);
		if(addr[j]=='*') return("Wildcards with aliases not supported (/ih).");
		if(addr[i] && addr[i]!='.' && k!=3)
			return("Missing parameter in IP address (/ih).");
		if(!addr[i] && k!=3) return("Premature ending of IP address (/ih).");
		if(j==i) return("Premature ending of IP address (/ih).");
		ip_alias_tbl[ip_alias_index].addr[k] = (char)atoi(&addr[j]);
		j=i+1;
	}

	*start=i;

	return("");
}

int	parse_options(int N_options, char *options[]) {
	int i;
	int j;
	int k;
	int status=0;
	int	hex;
	int	str_offset;
	char* status_msg="";
	char temp[BUFSIZ];

	for (i = 0; i < N_options; i++) {
		str_offset=0;
		switch(options[i][0]) {
		case '/':  								// implies option
		case '-':
			switch (options[i][1]) {
			case 'e':								// implies ethernet option
			case 'E':
				switch (options[i][2]) {
				case 's':							// ethernet source option
				case 'S':
					if (options[i][3] == '=') {
						for (j = 4; options[i][j] && options[i][j]!='('; j++) {
							if ((j-4)%3 < 2) {
								hex = hex2dec(options[i][j]);
								if (hex+1) {
									eth_src_tbl[eth_src_index].addr[(int)((j-4)/3)] +=
										((j-4)%3)?hex:hex << 4;
									eth_alias_tbl[eth_alias_index].addr[(int)((j-4)/3)] +=
										((j-4)%3)?hex:hex << 4;
								} else {
									status = 1;
									status_msg =
										"Parse error in ethernet source address (/es).";
								}
							}
						}
						if (j < ETHER_ADDR_1_EO) {
							status = 1;
							status_msg =
								"Premature ending of ethernet source address (/es).";
						}
						if (!status) {
							if (options[i][j]=='(') {
								if (options[i][strlen(options[i])-1]!=')')
									fprintf(stderr,"Warning:\n"
																 "  Missing closing ) or space in"
																 " alias name in ethernet source address"
																 " (/es).\n");
								farnearstrcpy(eth_alias_tbl[eth_alias_index].alias,
												&options[i][j+1]);
								eth_alias_tbl[eth_alias_index].alias[_fstrlen(
												eth_alias_tbl[eth_alias_index].alias)-1]='\0';
								eth_alias_index++;
							} else for (j=0; j < 6; j++) eth_alias_tbl[eth_alias_index].addr[j] = 0;
							eth_src_index++;
							nofilter_flag=0;
						}	else
							for (j=0; j < 6; j++)
								eth_src_tbl[eth_src_index].addr[j] =
								eth_alias_tbl[eth_alias_index].addr[j] = 0;
					} else {
						status = 1;
						status_msg = "Missing delimiter \'=\' to preceed ethernet source"
												 " address (/es).";
					}
					break;

				case 'd':							// ethernet destination option
				case 'D':
					if (options[i][3] != '\0') {
						for (j = 4; options[i][j] && options[i][j]!='('; j++) {
							if ((j-4)%3 < 2) {
								hex = hex2dec(options[i][j]);
								if (hex+1) {
									eth_dst_tbl[eth_dst_index].addr[(int)((j-4)/3)] +=
										((j-4)%3)?hex:hex << 4;
									eth_alias_tbl[eth_alias_index].addr[(int)((j-4)/3)] +=
										((j-4)%3)?hex:hex << 4;
								} else {
									status = 1;
									status_msg =
										"Parse error in ethernet destination address (/ed).";
								}
							}
						}
						if (j < ETHER_ADDR_1_EO) {
							status = 1;
							status_msg =
								"Premature ending of ethernet destination address (/ed).";
						}
						if (!status) {
							if (options[i][j]=='(') {
								if (options[i][strlen(options[i])-1]!=')')
									fprintf(stderr,"Warning:\n"
																 "  Missing closing ) or space in"
																 " alias name in ethernet destination"
																 " address (/ed).\n");
								farnearstrcpy(eth_alias_tbl[eth_alias_index].alias,&options[i][j+1]);
								eth_alias_tbl[eth_alias_index].alias[_fstrlen(eth_alias_tbl[eth_alias_index].alias)-1]=
									'\0';
								eth_alias_index++;
							} else for (j=0; j < 6; j++) eth_alias_tbl[eth_alias_index].addr[j] = 0;
							eth_dst_index++;
							nofilter_flag=0;
						}	else
							for (j=0; j < 6; j++)
								eth_dst_tbl[eth_dst_index].addr[j] =
								eth_alias_tbl[eth_alias_index].addr[j] = 0;
					} else {
						status = 1;
						status_msg = "Missing delimiter \'=\' to preceed ethernet"
												 " destination (/ed).";
					}
					break;

				case 'e':							// either ether source or destination
				case 'E':
					if (options[i][3]) {
						for (j = 4; options[i][j] && options[i][j]!='('; j++) {
							if ((j-4)%3 < 2) {
								hex = hex2dec(options[i][j]);
								if (hex+1) {
									eth_src_tbl[eth_src_index].addr[(int)((j-4)/3)] +=
										((j-4)%3)?hex:hex << 4;
									eth_dst_tbl[eth_dst_index].addr[(int)((j-4)/3)] +=
										((j-4)%3)?hex:hex << 4;
									eth_alias_tbl[eth_alias_index].addr[(int)((j-4)/3)] +=
										((j-4)%3)?hex:hex << 4;
								} else {
									status = 1;
									status_msg = "Parse error in ethernet address (/ee).";
								}
							}
						}
						if (j < ETHER_ADDR_1_EO) {
							status = 1;
							status_msg = "Premature ending of ethernet address (/ee).";
						}
						if (!status) {
							if (options[i][j]=='(') {
								if (options[i][strlen(options[i])-1]!=')')
									fprintf(stderr,"Warning:\n"
																 "  Missing closing ) or space in"
																 " alias name in ethernet address"
																 " (/ee).\n");
								farnearstrcpy(eth_alias_tbl[eth_alias_index].alias,&options[i][j+1]);
								eth_alias_tbl[eth_alias_index].alias[_fstrlen(eth_alias_tbl[eth_alias_index].alias)-1]=
									'\0';
								eth_alias_index++;
							} else for (j=0; j < 6; j++) eth_alias_tbl[eth_alias_index].addr[j] = 0;
							eth_src_index++;
							eth_dst_index++;
							nofilter_flag=0;
						} else for (j=0; j < 6; j++)
							eth_src_tbl[eth_src_index].addr[j] = eth_dst_tbl[eth_dst_index].addr[j] = eth_alias_tbl[eth_alias_index].addr[j] = 0;
					} else {
						status = 1;
						status_msg = "Missing delimiter \'=\' to preceed ethernet"
												 " address (/ee).";
					}
					break;

				case 'b':							// ether source AND destination must be
				case 'B':							//   together in packet (comma separated)
					if (options[i][3] != '\0') {
						for (j=4; options[i][j] && (j < 22); j++) {		// move past ether src
							if ((j-4)%3 < 2) {
								hex = hex2dec(options[i][j]);
								if (hex+1) {
									eth_src_tbl[eth_src_index].addr[(int)((j-4)/3)] +=
										((j-4)%3)?hex:hex << 4;
									eth_alias_tbl[eth_alias_index].addr[(int)((j-4)/3)] +=
										((j-4)%3)?hex:hex << 4;
								} else {
									status = 1;
									status_msg = "Parse error in ethernet address (/eb"
															 " source).";
								}
							}
						}
						if (j < ETHER_ADDR_1_EO) {
							status = 1;
							status_msg = "Premature ending of ethernet address"
													 " (/eb source).";
						}
						if (!status)
							if (options[i][j-1]=='(') {
								farnearstrcpy(eth_alias_tbl[eth_alias_index].alias,
											 &options[i][ETHER_ADDR_1_EO+1]);
								for (j=0; eth_alias_tbl[eth_alias_index].alias[j] &&
											eth_alias_tbl[eth_alias_index].alias[j]!=')'; j++);
								if (eth_alias_tbl[eth_alias_index].alias[j]!=')') {
									status = 1;
									status_msg = "Parse error in ethernet alias - missing ) "
															 " or destination address (/eb source).";
									for (j=0; j < 6; j++) eth_alias_tbl[eth_alias_index].addr[j] = 0;
								} else {
									eth_alias_tbl[eth_alias_index].alias[j] = '\0';
									str_offset = j+2;
									eth_alias_index++;
								}
							} else for (j=0; j < 6; j++) eth_alias_tbl[eth_alias_index].addr[j] = 0;
						if(!status)
							for (j=ETHER_ADDR_1_EO+1+str_offset; options[i][j] &&
									options[i][j]!='('; j++) {
								if ((j-(ETHER_ADDR_1_EO+1+str_offset))%3 < 2) {
									hex = hex2dec(options[i][j]);
									if (hex+1) {
										eth_dst_tbl[eth_dst_index].
											addr[(int)((j-(ETHER_ADDR_1_EO+1+str_offset))/3)]
											+= ((j-(ETHER_ADDR_1_EO+1+str_offset))%3)?hex:hex << 4;
										eth_alias_tbl[eth_alias_index].
											addr[(int)((j-(ETHER_ADDR_1_EO+1+str_offset))/3)]
											+= ((j-(ETHER_ADDR_1_EO+1+str_offset))%3)?hex:hex << 4;
									} else {
										status = 1;
										status_msg = "Parse error in ethernet address (/eb"
																 " destination).";
									}
								}
							}
						if ((j < ETHER_ADDR_2_EO+str_offset) &&
								(j >= ETHER_ADDR_1_EO+str_offset)) {
							status = 1;
							status_msg = "Premature ending of ethernet address"
													 " (/eb destination).";
						}
						if (!status)
							if (options[i][j]=='(') {
								farnearstrcpy(eth_alias_tbl[eth_alias_index].alias,
											 &options[i][ETHER_ADDR_2_EO+1+str_offset]);
								for (j=0; eth_alias_tbl[eth_alias_index].alias[j] &&
											eth_alias_tbl[eth_alias_index].alias[j]!=')'; j++);
								if (eth_alias_tbl[eth_alias_index].alias[j]!=')') {
									status = 1;
									status_msg = "Parse error in ethernet alias - possible "
															 " missing ) (/eb destination).";
									for (j=0; j < 6; j++) eth_alias_tbl[eth_alias_index].addr[j] = 0;
								} else {
									eth_alias_tbl[eth_alias_index].alias[j] = '\0';
									eth_alias_index++;
								}
							} else for (j=0; j < 6; j++) eth_alias_tbl[eth_alias_index].addr[j] = 0;
						if (!status) {
							eth_src_tbl[eth_src_index].logicop = eth_dst_tbl[eth_dst_index].logicop =
								ETHER_AND;
							eth_src_tbl[eth_src_index].index = eth_dst_index;
							eth_dst_tbl[eth_dst_index].index = eth_src_index;
							eth_dst_index++;
							eth_src_index++;
							nofilter_flag=0;
						} else for (j=0; j < 6; j++)
							eth_src_tbl[eth_src_index].addr[j] = eth_dst_tbl[eth_dst_index].addr[j] = 0;
					}	else {
						status = 1;
						status_msg = "Missing delimeter \'=\' to preceed ethernet"
												 " address (/eb).";
					}
					break;

				case 't':             // ethernet type
				case 'T':
					if (options[i][3] == '=') {
						if(strcmpi(&options[i][4],"IP")==0) {
							eth_type_tbl[eth_type_index++].type		   = ETHER_IP;
						} else
							if(strcmpi(&options[i][4],"ARP")==0) {
									eth_type_tbl[eth_type_index++].type  = ETHER_ARP;
							} else
								if(strcmpi(&options[i][4],"RARP")==0) {
									eth_type_tbl[eth_type_index++].type  = ETHER_RARP;
							} else {
								status = 1;
								status_msg = "Unsupported filter type (/et).";
							}
					} else {
						status = 1;
						status_msg = "Missing delimiter \'=\' to preceed ethernet type"
												 " (/et).";
					}
					break;

				case 'H':							// ethernet host table entry
				case 'h':
					if (options[i][3] == '=') {
						for (j = 4; options[i][j] && options[i][j]!='('; j++) {
							if ((j-4)%3 < 2) {
								hex = hex2dec(options[i][j]);
								if (hex+1) {
									eth_alias_tbl[eth_alias_index].addr[(int)((j-4)/3)] +=
										((j-4)%3)?hex:hex << 4;
								} else {
									status = 1;
									status_msg =
										"Parse error in ethernet host (alias) address (/eh).";
								}
							}
						}
						if (j < ETHER_ADDR_1_EO) {
							status = 1;
							status_msg =
								"Premature ending of ethernet host (alias) address (/eh).";
						}
						if (!status) {
							if (options[i][j]=='(') {
								if (options[i][strlen(options[i])-1]!=')')
									fprintf(stderr,"Warning:\n"
																 "  Missing closing ) or space in"
																 " alias name in ethernet host (alias)"
																 " address (/eh).\n");
								farnearstrcpy(eth_alias_tbl[eth_alias_index].alias,
												&options[i][j+1]);
								eth_alias_tbl[eth_alias_index].alias[_fstrlen(
												eth_alias_tbl[eth_alias_index].alias)-1]='\0';
								eth_alias_index++;
							}
						}	else for (j=0; j < 6; j++) eth_alias_tbl[eth_alias_index].addr[j] = 0;
					} else {
						status = 1;
						status_msg = "Missing delimiter \'=\' to preceed ethernet host"
												 " (alias) address (/eh).";
					}
					break;

				default:
					status = 1;
					status_msg = "Parse error following ethernet option (/e)";
				} 										// end e switch

				break; 								// break out of case e

			case 'i':								// internet protocol address option
			case 'I':
				switch(options[i][2]) {
				case 's':							// ip source
				case 'S':
					if (options[i][3] == '=') {
						j=4;
						status_msg = parse_ip_src_addr(options[i],&j);
						if(status_msg[0]) status = 1;

						if (!status) {
							if (options[i][j]=='(' && ip_src_tbl[ip_src_index].class=='d') {
								if (options[i][strlen(options[i])-1]!=')')
									fprintf(stderr,"Warning:\n"
																 "  Missing closing ) or space in"
																 " alias name in IP source address"
																 " (/is).\n");
								farnearstrcpy(ip_alias_tbl[ip_alias_index].alias,
												&options[i][j+1]);
								ip_alias_tbl[ip_alias_index].alias[_fstrlen(
												ip_alias_tbl[ip_alias_index].alias)-1]='\0';
								for (j=0; j<4; j++)
									ip_alias_tbl[ip_alias_index].addr[j] =
									ip_src_tbl[ip_src_index].addr[j];
								ip_alias_index++;
							}
							ip_src_index++;
							nofilter_flag=0;
						}	else
							for (j=0; j < 4; j++)
								ip_src_tbl[ip_src_index].addr[j] =
								ip_alias_tbl[ip_alias_index].addr[j] = 0;
					} else {
						status = 1;
						status_msg = "Missing delimiter \'=\' to preceed IP source"
												 " address (/is).";
					}

					break;
				case 'd':							// ip destination
				case 'D':
					if (options[i][3] == '=') {
						j=4;
						status_msg = parse_ip_dst_addr(options[i],&j);
						if(status_msg[0]) status = 1;

						if (!status) {
							if (options[i][j]=='(' && ip_dst_tbl[ip_dst_index].class=='d') {
								if (options[i][strlen(options[i])-1]!=')')
									fprintf(stderr,"Warning:\n"
																 "  Missing closing ) or space in"
																 " alias name in IP destination address"
																 " (/id).\n");
								farnearstrcpy(ip_alias_tbl[ip_alias_index].alias,
												&options[i][j+1]);
								ip_alias_tbl[ip_alias_index].alias[_fstrlen(
												ip_alias_tbl[ip_alias_index].alias)-1]='\0';
								for (j=0; j<4; j++)
									ip_alias_tbl[ip_alias_index].addr[j] =
									ip_dst_tbl[ip_dst_index].addr[j];
								ip_alias_index++;
							}
							ip_dst_index++;
							nofilter_flag=0;
						}	else
							for (j=0; j < 4; j++)
								ip_dst_tbl[ip_dst_index].addr[j] =
								ip_alias_tbl[ip_alias_index].addr[j] = 0;
					} else {
						status = 1;
						status_msg = "Missing delimiter \'=\' to preceed IP destination"
												 " address (/id).";
					}

					break;

				case 'e':
				case 'E':
					if (options[i][3] == '=') {
						j=4;
						status_msg = parse_ip_dst_addr(options[i],&j);
						if(status_msg[0]) status = 1;

						if (!status) {
							if (options[i][j]=='(' && ip_dst_tbl[ip_dst_index].class=='d') {
								if (options[i][strlen(options[i])-1]!=')')
									fprintf(stderr,"Warning:\n"
																 "  Missing closing ) or space in"
																 " alias name in IP address"
																 " (/ie).\n");
								farnearstrcpy(ip_alias_tbl[ip_alias_index].alias,
												&options[i][j+1]);
								ip_alias_tbl[ip_alias_index].alias[_fstrlen(
												ip_alias_tbl[ip_alias_index].alias)-1]='\0';
								for (j=0; j<4; j++)
									ip_alias_tbl[ip_alias_index].addr[j] =
									ip_dst_tbl[ip_dst_index].addr[j];
								ip_alias_index++;
							}
							for(j=0;j<4;j++) ip_src_tbl[ip_src_index].addr[j] =
																ip_dst_tbl[ip_dst_index].addr[j];
							ip_src_tbl[ip_src_index].class = ip_dst_tbl[ip_dst_index].class;
							ip_src_index++;
							ip_dst_index++;
							nofilter_flag=0;
						}	else
							for (j=0; j < 4; j++)
								ip_dst_tbl[ip_dst_index].addr[j] =
								ip_alias_tbl[ip_alias_index].addr[j] = 0;
					} else {
						status = 1;
						status_msg = "Missing delimiter \'=\' to preceed IP "
												 " address (/ie).";
					}

					break;
				case 'b':
				case 'B':
					if (options[i][3] == '=') {
						j=4;
						status_msg = parse_ip_src_addr(options[i],&j);
						if(status_msg[0]) status = 1;

						if (!status) {
							if (options[i][j]=='(' && ip_src_tbl[ip_src_index].class=='d') {

								farnearstrcpy(ip_alias_tbl[ip_alias_index].alias,
											 &options[i][j+1]);
								for (k=0; ip_alias_tbl[ip_alias_index].alias[k] &&
											ip_alias_tbl[ip_alias_index].alias[k]!=')'; k++);
								if (ip_alias_tbl[ip_alias_index].alias[k]!=')') {
									status = 1;
									status_msg = "Parse error in IP alias - missing ) "
															 " or destination address (/ib source).";
									for (k=0; k < 4; k++) ip_alias_tbl[ip_alias_index].addr[k] = 0;
								} else {
									j+=k+2;
									ip_alias_tbl[ip_alias_index].alias[k] = '\0';
									for (k=0; k<4; k++)
										ip_alias_tbl[ip_alias_index].addr[k] =
										ip_src_tbl[ip_src_index].addr[k];
									ip_alias_index++;
								}
							}
						}	else
							for (k=0; k < 4; k++)
								ip_src_tbl[ip_src_index].addr[k] =
								ip_alias_tbl[ip_alias_index].addr[k] = 0;
					} else {
						status = 1;
						status_msg = "Missing delimiter \'=\' to preceed IP source"
												 " address (/ib).";
					}

					if (!status && options[i][j] == ',') {
						j++;
						status_msg = parse_ip_dst_addr(options[i],&j);
						if(status_msg[0]) status = 1;

						if (!status) {
							if (options[i][j]=='(' && ip_dst_tbl[ip_dst_index].class=='d') {
								if (options[i][strlen(options[i])-1]!=')')
									fprintf(stderr,"Warning:\n"
																 "  Missing closing ) or space in"
																 " alias name in IP destination address"
																 " (/ib).\n");
								farnearstrcpy(ip_alias_tbl[ip_alias_index].alias,
												&options[i][j+1]);
								ip_alias_tbl[ip_alias_index].alias[_fstrlen(
												ip_alias_tbl[ip_alias_index].alias)-1]='\0';
								for (j=0; j<4; j++)
									ip_alias_tbl[ip_alias_index].addr[j] =
									ip_dst_tbl[ip_dst_index].addr[j];
								ip_alias_index++;
							}
							ip_src_tbl[ip_src_index].logicop =
								ip_dst_tbl[ip_dst_index].logicop = IP_AND;
							ip_src_tbl[ip_src_index].index = ip_dst_index;
							ip_dst_tbl[ip_dst_index].index = ip_src_index;
							ip_src_index++;
							ip_dst_index++;
							nofilter_flag=0;
						}	else
							for (j=0; j < 4; j++)
								ip_dst_tbl[ip_dst_index].addr[j] =
								ip_alias_tbl[ip_alias_index].addr[j] = 0;
					} else {
						status = 1;
						status_msg = "Missing delimiter \',\' to preceed IP destination"
												 " address (/ib).";
					}


					break;
				case 'h':
				case 'H':
					if (options[i][3] == '=') {
						j=4;
						status_msg = parse_ip_alias_addr(options[i],&j);
						if(status_msg[0]) status = 1;

						if (!status) {
							if (options[i][j]=='(') {
								if (options[i][strlen(options[i])-1]!=')')
									fprintf(stderr,"Warning:\n"
																 "  Missing closing ) or space in"
																 " alias name in IP source address"
																 " (/is).\n");
								farnearstrcpy(ip_alias_tbl[ip_alias_index].alias,
												&options[i][j+1]);
								ip_alias_tbl[ip_alias_index].alias[_fstrlen(
												ip_alias_tbl[ip_alias_index].alias)-1]='\0';
								ip_alias_index++;
							}
						}	else
							for (j=0; j < 4; j++)
								ip_alias_tbl[ip_alias_index].addr[j] = 0;
					} else {
						status = 1;
						status_msg = "Missing delimiter \'=\' to preceed IP "
												 " address (/ih).";
					}


					break;
				case 't':
				case 'T':
					if (options[i][3] == '=') {
						if(strcmpi(&options[i][4],"ICMP")==0) {
							ip_type_tbl[ip_type_index++].type	= IP_ICMP;
						} else
							if(strcmpi(&options[i][4],"TCP")==0) {
									ip_type_tbl[ip_type_index++].type = IP_TCP;
							} else
								if(strcmpi(&options[i][4],"UDP")==0) {
										ip_type_tbl[ip_type_index++].type = IP_UDP;
								} else
									if(strcmpi(&options[i][4],"MUX")==0) {
											ip_type_tbl[ip_type_index++].type = IP_MUX;
									} else {
										status = 1;
										status_msg = "Unsupported filter type (/it).";
									}
					} else {
						status = 1;
						status_msg = "Missing delimiter \'=\' to preceed IP type"
												 " (/it).";
					}
					break;
				default:
					status = 1;
					status_msg = "Parse error following IP option (/i)";
				}
				break;
			case 't':
			case 'T':
				switch(options[i][2]) {
				case 't':							// ip source
				case 'T':
					if (options[i][3] == '=') {
						ip_type_tbl[ip_type_index++].type = IP_TCP;
						if(strcmpi(&options[i][4],"ECHO")==0) {
							tcp_type_tbl[tcp_type_index++].type	= TCP_ECHO;
						} else
							if(strcmpi(&options[i][4],"NETSTAT")==0) {
									tcp_type_tbl[tcp_type_index++].type = TCP_NETSTAT;
							} else
								if(strcmpi(&options[i][4],"CHARGEN")==0) {
										tcp_type_tbl[tcp_type_index++].type = TCP_CHARGEN;
								} else
									if(strcmpi(&options[i][4],"FTP_DATA")==0) {
											tcp_type_tbl[tcp_type_index++].type = TCP_FTP_DATA;
									} else
										if(strcmpi(&options[i][4],"FTP")==0) {
												 tcp_type_tbl[tcp_type_index++].type = TCP_FTP;
									} else
										if(strcmpi(&options[i][4],"SSH")==0) {
												 tcp_type_tbl[tcp_type_index++].type = TCP_SSH;
									} else
										if(strcmpi(&options[i][4],"TELNET")==0) {
												 tcp_type_tbl[tcp_type_index++].type = TCP_TELNET;
									} else
										if(strcmpi(&options[i][4],"SMTP")==0) {
												 tcp_type_tbl[tcp_type_index++].type = TCP_SMTP;
									} else
										if(strcmpi(&options[i][4],"TIME")==0) {
												 tcp_type_tbl[tcp_type_index++].type = TCP_TIME;
									} else
										if(strcmpi(&options[i][4],"NAMESERVER")==0) {
												 tcp_type_tbl[tcp_type_index++].type = TCP_NAMESERVER;
									} else
										if(strcmpi(&options[i][4],"WHOIS")==0) {
												 tcp_type_tbl[tcp_type_index++].type = TCP_WHOIS;
									} else
										if(strcmpi(&options[i][4],"BOOTPS")==0) {
												 tcp_type_tbl[tcp_type_index++].type = TCP_BOOTPS;
									} else
										if(strcmpi(&options[i][4],"BOOTPC")==0) {
												 tcp_type_tbl[tcp_type_index++].type = TCP_BOOTPC;
									} else
										if(strcmpi(&options[i][4],"GOPHER")==0) {
												 tcp_type_tbl[tcp_type_index++].type = TCP_GOPHER;
									} else
										if(strcmpi(&options[i][4],"FINGER")==0) {
												 tcp_type_tbl[tcp_type_index++].type = TCP_FINGER;
									} else
										if(strcmpi(&options[i][4],"WWW")==0) {
												 tcp_type_tbl[tcp_type_index++].type = TCP_WWW;
									} else
										if(strcmpi(&options[i][4],"KERBEROS")==0) {
												 tcp_type_tbl[tcp_type_index++].type = TCP_KERBEROS;
									} else
										if(strcmpi(&options[i][4],"RTELNET")==0) {
												 tcp_type_tbl[tcp_type_index++].type = TCP_RTELNET;
									} else
										if(strcmpi(&options[i][4],"POP_2")==0) {
												 tcp_type_tbl[tcp_type_index++].type = TCP_POP_2;
									} else
										if(strcmpi(&options[i][4],"POP_3")==0) {
												 tcp_type_tbl[tcp_type_index++].type = TCP_POP_3;
									} else
										if(strcmpi(&options[i][4],"SUNRPC")==0) {
												 tcp_type_tbl[tcp_type_index++].type = TCP_SUNRPC;
									} else
										if(strcmpi(&options[i][4],"NNTP")==0) {
												 tcp_type_tbl[tcp_type_index++].type = TCP_NNTP;
									} else
										if(strcmpi(&options[i][4],"NTP")==0) {
												 tcp_type_tbl[tcp_type_index++].type = TCP_NTP;
									} else
										if(strcmpi(&options[i][4],"NETBIOS_NS")==0) {
												 tcp_type_tbl[tcp_type_index++].type = TCP_NETBIOS_NS;
									} else
										if(strcmpi(&options[i][4],"NETBIOS_DGM")==0) {
												 tcp_type_tbl[tcp_type_index++].type = TCP_NETBIOS_DGM;
									} else
										if(strcmpi(&options[i][4],"NETBIOS_SSN")==0) {
												 tcp_type_tbl[tcp_type_index++].type = TCP_NETBIOS_SSN;
									} else
										if(strcmpi(&options[i][4],"XDMCP")==0) {
												 tcp_type_tbl[tcp_type_index++].type = TCP_XDMCP;
									} else
										if(strcmpi(&options[i][4],"NEXTSTEP")==0) {
												 tcp_type_tbl[tcp_type_index++].type = TCP_NEXTSTEP;
									} else
										if(strcmpi(&options[i][4],"IRC")==0) {
												 tcp_type_tbl[tcp_type_index++].type = TCP_IRC;
									} else
										if(strcmpi(&options[i][4],"IPX")==0) {
												 tcp_type_tbl[tcp_type_index++].type = TCP_IPX;
									} else
										if(strcmpi(&options[i][4],"EXEC")==0) {
												 tcp_type_tbl[tcp_type_index++].type = TCP_EXEC;
									} else
										if(strcmpi(&options[i][4],"LOGIN")==0) {
												 tcp_type_tbl[tcp_type_index++].type = TCP_LOGIN;
									} else
										if(strcmpi(&options[i][4],"SHELL")==0) {
												 tcp_type_tbl[tcp_type_index++].type = TCP_SHELL;
									} else
										if(strcmpi(&options[i][4],"PRINTER")==0) {
												 tcp_type_tbl[tcp_type_index++].type = TCP_PRINTER;
									} else
										if(strcmpi(&options[i][4],"UUCP")==0) {
												 tcp_type_tbl[tcp_type_index++].type = TCP_UUCP;
									} else
										if(strcmpi(&options[i][4],"KLOGIN")==0) {
												 tcp_type_tbl[tcp_type_index++].type = TCP_KLOGIN;
									} else
										if(strcmpi(&options[i][4],"LSHELL")==0) {
												 tcp_type_tbl[tcp_type_index++].type = TCP_KSHELL;
									} else
										if(strcmpi(&options[i][4],"KERBEROS_ADM")==0) {
												 tcp_type_tbl[tcp_type_index++].type = TCP_KERBEROS_ADM;
									} else
										if(strcmpi(&options[i][4],"KERBEROS4")==0) {
												 tcp_type_tbl[tcp_type_index++].type = TCP_KERBEROS4;
									} else
										if(strcmpi(&options[i][4],"KERBEROS_MASTER")==0) {
												 tcp_type_tbl[tcp_type_index++].type = TCP_KERBEROS_MASTER;
									} else
										if(strcmpi(&options[i][4],"KRB_PROP")==0) {
												 tcp_type_tbl[tcp_type_index++].type = TCP_KRB_PROP;
									} else
										if(strcmpi(&options[i][4],"KRBUPDATE")==0) {
												 tcp_type_tbl[tcp_type_index++].type = TCP_KRBUPDATE;
									} else
										if(strcmpi(&options[i][4],"KPASSWD")==0) {
												 tcp_type_tbl[tcp_type_index++].type = TCP_KPASSWD;
									} else
										if(strcmpi(&options[i][4],"KPOP")==0) {
												 tcp_type_tbl[tcp_type_index++].type = TCP_KPOP;
									} else
										if(strcmpi(&options[i][4],"KNETD")==0) {
												 tcp_type_tbl[tcp_type_index++].type = TCP_KNETD;
									} else
										if(strcmpi(&options[i][4],"EKLOGIN")==0) {
												 tcp_type_tbl[tcp_type_index++].type = TCP_EKLOGIN;
									} else
										if(strcmpi(&options[i][4],"CFINGER")==0) {
												 tcp_type_tbl[tcp_type_index++].type = TCP_CFINGER;
									} else
										if(isdigit(options[i][4])) {
											tcp_type_tbl[tcp_type_index++].type =
												atoi(&options[i][4]);
									} else {
										status = 1;
										status_msg = "Unsupported filter type (/tt).";
									}
					} else {
						status = 1;
						status_msg = "Missing delimiter \'=\' to preceed TCP type"
												 " name (/tt).";
					}

					break;
				default:
					status = 1;
					status_msg = "Parse error following IP option (/i)";
				}
				break;

			case 'c':
			case 'C':
				character_mode = 1;
				break;

			case 'h':
			case 'H':
			case '?':
				usage();
				status = 1;
				break; 								// break out of case h,?

			case 'v':
			case 'V':
				verbose_mode = 1;
				break;

			case 'l':
			case 'L':
				if (options[i][2] == '=') {
					if(log_file != stdout) fclose(log_file);

					log_file = fopen(&options[i][3],"w");
					if(!log_file) {
						status = 1;
						fprintf(stderr, "Unable to open file \"%s\" for logging.\n",
										&options[i][3]);
					} else
						fprintf(stdout, "Logging to file \"%s\".\n", &options[i][3]);
				} else {
					status = 1;
					status_msg = "Missing delimiter \'=\' to preceed log file"
											 " (/l).";
				}

				break; 								// break out of case l

			default:
				status = 1;
				fprintf(stderr,"Unrecognized option /%c%c.\n",options[i][0],
								options[i][1]);
			}

			break; 									// break for - or /

		case '#':									// verbose remark
			fprintf(stdout,"-- %s\n",&options[i][1]);
			if(log_file != stdout) fprintf(log_file,"-- %s\n",&options[i][1]);
			break;

		case ';':									// silent remark
		case '\n':
		case '\0':
			break;

		default:
			status = read_optfile((char *)options[i]);
			if(status) fprintf(stderr,"Error in option file \"%s\".\n",
				options[i]);
		} 												// unrecognized symbol
	} 													// end for loop

	if (status_msg && status) fprintf(stderr,"%s\n",status_msg);
	return(status);
}

int read_optfile(char *filespec) {
	int   status=0;
	char *option[MAXOPT];				// arbitrarily sized option list
	int		index;
	int		i;
	FILE *fp;
	char *filename;

	for(i=0; filespec[i] && filespec[i]!='.'; i++);		// check to see if
	if(filespec[i]=='.') filename = filespec;					//  default extension
	else {																						//  is required
		filename = (char *)malloc(sizeof(char)*(strlen(filespec)+5));
		if(!filename) {
			fprintf(stderr, "Could not allocate memory for filename in read_optfile.\n");
			exit(1);
		}
		for(i=0; filespec[i]; i++) filename[i]=filespec[i];
		filename[i]='.';
		filename[i+1]='o';
		filename[i+2]='p';
		filename[i+3]='t';
		filename[i+4]='\0';
	}

	fp = fopen(filename,"r");

	if(!fp) {
		status=1;
		fprintf(stderr,"Could not open file \"%s\".\n", filename);
	} else {
		fprintf(stdout,"\nReading options from file \"%s\"", filename);
		if(verbose_mode) fprintf(stdout,"...\n");

		for(i=0; i<MAXOPT; i++) {
			option[i] = (char *)malloc(sizeof(char)*OPTBUFSIZ);
			if(!option[i]) {
				fprintf(stderr, "Could not allocate memory for option[%d] in read_optfile.\n",i);
				exit(1);
			}
		}
		for(index=0; (index < MAXOPT) && !status &&
				fgets(option[index],OPTBUFSIZ,fp); index++) {

			for(i=0; (i<OPTBUFSIZ) && option[index][i] &&
					isspace(option[index][i]); i++);
			option[index] = &option[index][i];

			for(i=0; (i<OPTBUFSIZ) && option[index][i] &&
					option[index][i]!='\n'; i++);             // remove \n
			if(i<OPTBUFSIZ) option[index][i]='\0';

			if(!verbose_mode) fprintf(stdout,".");
			else fprintf(stdout,"%s\n",option[index]);

			for(i=0; (i<OPTBUFSIZ) && option[index][i] &&	// remove anything after
					!isspace(option[index][i]); i++);					//   whitespace

			if(i<OPTBUFSIZ && !(option[index][0] == ';' || option[index][0] == '#'))
				 option[index][i] = '\0';										//   except when comment

			if(i==OPTBUFSIZ) {
				status = 1;
				fprintf(stderr,"Option too long on line %i of file \"%s\".\n",
								index,filename);
			}
		}
		fclose(fp);
		if(!verbose_mode) fprintf(stdout,"\n");
		if(!status)	status = parse_options(index,option);

		if(!status && index==MAXOPT)
			fprintf(stderr,"Warning: Options possibly truncated past line %i of"
										 " file \"%s\".\n",index,filename);

		for(i=0; i<MAXOPT; i++) free(option[i]);
	}

	return(status);
}