/**********************************************************************
* procedure : Final Semester Project
*
*      file : snoop.c
*
*       			(C) 1996, Brendan D. Donahe and Jerome C. Parks,
*							all rights reserved
*
*   project : snoop.exe
*
*   purpose : Main code: To filter TCP/IP and Ethernet packets
*
*   actions : 1. Definition of all global variables
*							2. Initialize all buffers and tables
*							3. Print banner and time stamp
*							4. Initialize and configure packet driver to permiscious mode
*							5. Execute circular buffers of filler and printing data
*								 returned from packet driver
*							6. Print closing statistics and banner and exit
*
*     usage : snoop {options(s)} {filename(s)}
*							snoop -h  prints command line options
*
* exit code : 1 implies error status, 0 imples successful
*
*   authors : Brendan D. Donahe and
*							Jerome C. Parks,
*							New Mexico State University, EE 464
*
*  language : Borland International C/C++, version 2.0
*							plus corresponding assembler and linker
*
*   history : December 13, 1996,  Verion 1.0
*
***********************************************************************/


#include <stdio.h>
#include <dos.h>									// for gettime and getdate
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <alloc.h>
#include <conio.h>

#define  SNOOP_MAIN
#include "pktdrvr.h"
#include "ether.h"
#include "ip.h"
#include "tcp.h"
#include "snoop.h"

volatile PKTBUF far        *receive_buffer;
volatile char               receiver_busy  = 0;
volatile unsigned long int  discarded_pkts = 0;

unsigned long int			      received_pkts = 0;
unsigned long int						filtered_out_pkts = 0;

ETH_ADDR_TBL far					 *eth_src_tbl;
ETH_ADDR_TBL far					 *eth_dst_tbl;
ETH_TYPE_TBL far					 *eth_type_tbl;
ETH_ALIAS_TBL far					 *eth_alias_tbl;
int 									      eth_src_index=0;
int 									      eth_dst_index=0;
int													eth_type_index=0;
int													eth_alias_index=0;

IP_ADDR_TBL  far 					 *ip_src_tbl;
IP_ADDR_TBL  far 					 *ip_dst_tbl;
IP_TYPE_TBL  far 					 *ip_type_tbl;
IP_ALIAS_TBL far 					 *ip_alias_tbl;
int 												ip_src_index;
int 												ip_dst_index;
int 												ip_type_index;
int 												ip_alias_index;

TCP_PORT_TBL  far 				 *tcp_src_tbl;
TCP_PORT_TBL  far 				 *tcp_dst_tbl;
TCP_TYPE_TBL  far 				 *tcp_type_tbl;
TCP_ALIAS_TBL far 				 *tcp_alias_tbl;
int 												tcp_src_index;
int 												tcp_dst_index;
int 												tcp_type_index;
int 												tcp_alias_index;


char												nofilter_flag=1;

char												character_mode=0;

char												verbose_mode=0;

FILE											 *log_file;

int main(int argc, char *argv[])
{
	int       	 pktdrvint;
	int		       i;
	int		       return_code = 0;
	int		       handle;
	int					 direct_addr_video;
	DRVR_INFO 	 info;
	ACCESS_TYPE  access;
	PKTBUF far	*current_buffer;
	PKTBUF far  *first_buffer;
	struct time	 t;
	struct date	 d;
	char far    *tcp_begin;
	int					 j;

	log_file = stdout;

	gettime(&t);
	getdate(&d);

	fprintf(stdout, "\nSnoop: (C)1996 Brendan D. Donahe and Jerome C. Parks\n");

	fprintf(stdout, "Free memory heap: %lu kbytes\n", farcoreleft()/(long int)1024);

	eth_clear_table();														// clear out source and dest
	ip_clear_table();
	tcp_clear_table();

	if(!parse_options(argc-1,&argv[1])) {

		fprintf(stdout, "\nExecution initiated %02d:%02d:%02d.%02d on ",
						t.ti_hour,t.ti_min,t.ti_sec,t.ti_hund);
		fprintf(stdout, "%02d/%02d/%02d.\n\n",d.da_mon,d.da_day,d.da_year);

		if(log_file!=stdout) {
			fprintf(log_file, "Execution initiated %02d:%02d:%02d.%02d on ",
							t.ti_hour,t.ti_min,t.ti_sec,t.ti_hund);
			fprintf(log_file, "%02d/%02d/%02d.\n\n",d.da_mon,d.da_day,d.da_year);
		}

		build_buffers(&first_buffer);         	// building link list of N_BUFS

		receive_buffer = current_buffer = first_buffer;
																						// set up receiver to point to
																						// buffer to display next

		pktdrvint = pkt_find();       					// find packet driver interrupt

		if(pktdrvint) {
			fprintf(log_file,"Packet driver:\t");

			if(get_drvrinfo(log_file, pktdrvint, (DRVR_INFO far *)(&info))) {

				handle = access_type_all(pktdrvint, (DRVR_INFO far *)(&info),
																(ACCESS_TYPE far *)(&access));

				fprintf(log_file,"\t\t    Interrupt -  0x%x\n", (int)pktdrvint);

				if(handle >= 0) {
					fprintf(log_file,"\nReceiving packets...\n");
					if (log_file!=stdout) fprintf(stdout,"\nReceiving packets...       <esc> key terminates execution\n");

					set_recv(pktdrvint, handle, RCVR_PERMISCUOUS);

					if(log_file!=stdout) display_stats(stdout);

					for(;;) {
						if(kbhit())
							if(getch() == 27)
								break;

						if(current_buffer->pktsize_flag) {
							if((eth_addr_filter((ETH_HEADER far *)
																	(current_buffer->buffer)) ||
								 ip_addr_filter((IP_HEADER far *)
																	(current_buffer->buffer)) ||
								 nofilter_flag)) {

								type_swap((unsigned int far *)&(((ETH_HEADER far *)
																				(current_buffer->buffer))->type));

								if(((ETH_HEADER far *)(current_buffer->buffer))->type ==
										ETHER_IP) {

									type_swap((unsigned int far *)&(((IP_HEADER far *)
										(current_buffer->buffer))->length));

									if(((IP_HEADER far *)(current_buffer->buffer))->type ==
											IP_TCP) {
										tcp_begin = (((((IP_HEADER far *)
																(current_buffer->buffer))->vers_IHL) & 0x0F )
																 << 2) + (char far *)&(((IP_HEADER far *)
																 (current_buffer->buffer))->vers_IHL);
										type_swap((unsigned int far *)&(((TCP_HEADER far *)
											(tcp_begin))->src));
										type_swap((unsigned int far *)&(((TCP_HEADER far *)
											(tcp_begin))->dst));
									}
								}

								if(eth_type_filter((ETH_HEADER far *)
																 (current_buffer->buffer))) {
									if(ip_type_filter((IP_HEADER far *)
																	(current_buffer->buffer))) {
										if(tcp_type_filter((TCP_HEADER far *)tcp_begin)) {
																									// display header output
											if(!character_mode)
												display_timestamp();

											if(!character_mode)
												eth_header((ETH_HEADER far *)(current_buffer->buffer),
															current_buffer->pktsize_flag);

											if(((ETH_HEADER far *)(current_buffer->buffer))->type ==
													ETHER_IP) {
												if(!character_mode)
													ip_header((IP_HEADER far *)(current_buffer->buffer));

												if(((IP_HEADER far *)(current_buffer->buffer))->type ==
													IP_TCP) {

													if(character_mode) {
														if(isalnum(tcp_begin[20]))
															fprintf(log_file, "%c",tcp_begin[20]);
														else fprintf(log_file," ");
													}
													if(!character_mode)
														tcp_header((TCP_HEADER far *)(tcp_begin));
													((TCP_HEADER far *)tcp_begin)->src = 0;
													((TCP_HEADER far *)tcp_begin)->dst = 0;
												}
											}
											received_pkts++;	// increment received_pkts counter
											if(!character_mode)
												fprintf(log_file,"\n");
										} else filtered_out_pkts++;
									} else filtered_out_pkts++;
								} else filtered_out_pkts++;
								if(log_file!=stdout) current_stats();

							} else filtered_out_pkts++;
							current_buffer->pktsize_flag = 0;	// reset pktsize flag

							current_buffer = current_buffer->next; // look at next buffer

						}										// end pkt size flag wait
					} 										// end for loop
					return_code = reltype(pktdrvint, handle);
					if(character_mode) fprintf(log_file,"\n");
					fprintf(log_file,"Releasing receiver handle (%d).\n",
									handle);
					display_stats(log_file);
				} else {
					fprintf(stderr,"Access handle error %d\n", handle);
					return_code = 1;
				}												// end handle check
			} else {
				fprintf(stderr,"Not installed.\n");
				return_code = 1;
			}													// end get_drvrinfo check

		}	else {
			fprintf(stderr,"Packet driver not found.\n");
			return_code = 1;
		}														// end pkt drive installation check
	}															// end option parsing check

	if(log_file != stdout) fclose(log_file);

	gettime(&t);
	getdate(&d);

	fprintf(stdout, "\nSnoop: (C)1996 Brendan D. Donahe and Jerome C. Parks\n");

	fprintf(stdout, "Free memory heap: %lu kbytes\n", farcoreleft()/(long int)1024);

	fprintf(stdout, "\nExecution ended %02d:%02d:%02d.%02d on ",
					t.ti_hour,t.ti_min,t.ti_sec,t.ti_hund);
	fprintf(stdout, "%02d/%02d/%02d.\n\n",d.da_mon,d.da_day,d.da_year);
	if(log_file!=stdout) {
		fprintf(log_file, "\nExecution ended %02d:%02d:%02d.%02d on ",
						t.ti_hour,t.ti_min,t.ti_sec,t.ti_hund);
		fprintf(log_file, "%02d/%02d/%02d.\n",d.da_mon,d.da_day,d.da_year);
	}

	return(return_code);
}																// end main
