/**********************************************************************
* procedure : Final Semester Project
*
*      file : tcphead.c
*
*       			(C) 1996, Brendan D. Donahe and Jerome C. Parks,
*							all rights reserved
*
*   project : snoop.exe
*
*   purpose : Displays TCP header to log file
*
*   actions : 1. Converts to type strings if possible
*							2.  Displays TCP header
*
*     usage : void tcp_header(TCP_HEADER far *header);
*
* exit code : None
*
*   authors : Brendan D. Donahe and
*							Jerome C. Parks,
*							New Mexico State University, EE 464
*
*  language : Borland International C/C++, version 2.0
*							plus corresponding assembler and linker
*
*   history : December 13, 1996,  Verion 1.0
*
***********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <dos.h>		// for gettime and getdate

#include "pktdrvr.h"
#include "ether.h"
#include "ip.h"
#include "tcp.h"
#include "snoop.h"

void tcp_header(TCP_HEADER far *header) {
	char far *srcport;
	char far *dstport;

	srcport = tcp_port_type(header->src);
	if(srcport)
		fprintf(log_file,"  TCP:             src %-17Fs  dst ",srcport);
	else
		fprintf(log_file,"  TCP:             src %-17u  dst ",header->src);

	dstport = tcp_port_type(header->dst);
	if(dstport)
		fprintf(log_file,"%-17Fs\n",dstport);
	else
		fprintf(log_file,"%-17u\n",header->dst);

}

char far *tcp_port_type(unsigned int portnum) {
	char far *portname;

	switch(portnum) {
	case TCP_TCPMUX:
		portname = "TCPMUX";
		break;
	case TCP_ECHO:
		portname = "ECHO";
		break;
	case TCP_DISCARD:
		portname = "DISCARD";
		break;
	case TCP_SYSTAT:
		portname = "SYSTAT";
		break;
	case TCP_DAYTIME:
		portname = "DAYTIME";
		break;
	case TCP_NETSTAT:
		portname = "NETSTAT";
		break;
	case TCP_QOTD:
		portname = "QOTD";
		break;
	case TCP_MSP:
		portname = "MSP";
		break;
	case TCP_CHARGEN:
		portname = "CHARGEN";
		break;
	case TCP_FTP_DATA:
		portname = "FTP_DATA";
		break;
	case TCP_FTP:
		portname = "FTP";
		break;
	case TCP_SSH:
		portname = "SSH";
		break;
	case TCP_TELNET:
		portname = "TELNET";
		break;
	case TCP_SMTP:
		portname = "SMTP";
		break;
	case TCP_TIME:
		portname = "TIME";
		break;
	case TCP_NAMESERVER:
		portname = "NAMESERVER";
		break;
	case TCP_WHOIS:
		portname = "WHOIS";
		break;
	case TCP_RE_MAIL_CK:
		portname = "RE_MAIL_CK";
		break;
	case TCP_DOMAIN:
		portname = "DOMAIN";
		break;
	case TCP_MTP:
		portname = "MTP";
		break;
	case TCP_BOOTPS:
		portname = "BOOTPS";
		break;
	case TCP_BOOTPC:
		portname = "BOOTPC";
		break;
	case TCP_GOPHER:
		portname = "GOPHER";
		break;
	case TCP_RJE:
		portname = "RJE";
		break;
	case TCP_FINGER:
		portname = "FINGER";
		break;
	case TCP_WWW:
		portname = "WWW";
		break;
	case TCP_LINK:
		portname = "LINK";
		break;
	case TCP_KERBEROS:
		portname = "KERBEROS";
		break;
	case TCP_SUPDUP:
		portname = "SUPDUP";
		break;
	case TCP_HOSTNAMES:
		portname = "HOSTNAMES";
		break;
	case TCP_ISO_TSAP:
		portname = "ISO_TSAP";
		break;
	case TCP_CSNET_NS:
		portname = "CSNET_NS";
		break;
	case TCP_RTELNET:
		portname = "RTELNET";
		break;
	case TCP_POP_2:
		portname = "POP_2";
		break;
	case TCP_POP_3:
		portname = "POP_3";
		break;
	case TCP_SUNRPC:
		portname = "SUNRPC";
		break;
	case TCP_AUTH:
		portname = "AUTH";
		break;
	case TCP_SFTP:
		portname = "SFTP";
		break;
	case TCP_UUCP_PATH:
		portname = "UUCP_PATH";
		break;
	case TCP_NNTP:
		portname = "NNTP";
		break;
	case TCP_NTP:
		portname = "NTP";
		break;
	case TCP_NETBIOS_NS:
		portname = "NETBIOS_NS";
		break;
	case TCP_NETBIOS_DGM:
		portname = "NETBIOS_DGM";
		break;
	case TCP_NETBIOS_SSN:
		portname = "NETBIOS_SSN";
		break;
	case TCP_IMAP2:
		portname = "IMAP2";
		break;
	case TCP_CMIP_MAN:
		portname = "CMIP_MAN";
		break;
	case TCP_CMIP_AGENT:
		portname = "CMIP_AGENT";
		break;
	case TCP_XDMCP:
		portname = "XDMCP";
		break;
	case TCP_NEXTSTEP:
		portname = "NEXTSTEP";
		break;
	case TCP_BGP:
		portname = "BGP";
		break;
	case TCP_PROSPERO:
		portname = "PROSPERO";
		break;
	case TCP_IRC:
		portname = "IRC";
		break;
	case TCP_SMUX:
		portname = "SMUX";
		break;
	case TCP_AT_RTMP:
		portname = "AT_RTMP";
		break;
	case TCP_AT_NBP:
		portname = "AT_NBP";
		break;
	case TCP_AT_ECHO:
		portname = "AT_ECHO";
		break;
	case TCP_AT_ZIS:
		portname = "AT_ZIS";
		break;
	case TCP_Z3950:
		portname = "Z3950";
		break;
	case TCP_IPX:
		portname = "IPX";
		break;
	case TCP_IMAP3:
		portname = "IMAP3";
		break;
	case TCP_ULISTSERV:
		portname = "ULISTSERV";
		break;
	case TCP_EXEC:
		portname = "EXEC";
		break;
	case TCP_LOGIN:
		portname = "LOGIN";
		break;
	case TCP_SHELL:
		portname = "SHELL";
		break;
	case TCP_PRINTER:
		portname = "PRINTER";
		break;
	case TCP_TEMPO:
		portname = "TEMPO";
		break;
	case TCP_COURIER:
		portname = "COURIER";
		break;
	case TCP_CONFERENCE:
		portname = "CONFERENCE";
		break;
	case TCP_NETNEWS:
		portname = "NETNEWS";
		break;
	case TCP_UUCP:
		portname = "UUCP";
		break;
	case TCP_REMOTEFS:
		portname = "REMOTEFS";
		break;
	case TCP_KLOGIN:
		portname = "KLOGIN";
		break;
	case TCP_KSHELL:
		portname = "KSHELL";
		break;
	case TCP_KERBEROS_ADM:
		portname = "KERBEROS_ADM";
		break;
	case TCP_WEBSTER:
		portname = "WEBSTER";
		break;
	case TCP_INGRESLOCK:
		portname = "INGRESLOCK";
		break;
	case TCP_PROSPERO_NP:
		portname = "PROSPERO_NP";
		break;
	case TCP_RFE:
		portname = "RFE";
		break;
	case TCP_BBS:
		portname = "BBS";
		break;
	case TCP_KERBEROS4:
		portname = "KERBEROS4";
		break;
	case TCP_KERBEROS_MASTER:
		portname = "KERBEROS_MASTER";
		break;
	case TCP_KRB_PROP:
		portname = "KRB_PROP";
		break;
	case TCP_KRBUPDATE:
		portname = "KRBUPDATE";
		break;
	case TCP_KPASSWD:
		portname = "KPASSWD";
		break;
	case TCP_KPOP:
		portname = "KPOP";
		break;
	case TCP_KNETD:
		portname = "KNETD";
		break;
	case TCP_EKLOGIN:
		portname = "EKLOGIN";
		break;
	case TCP_SUPFILESRV:
		portname = "SUPFILESRV";
		break;
	case TCP_SUPFILEDBG:
		portname = "SUPFILEDBG";
		break;
	case TCP_RMTCFG:
		portname = "RMTCFG";
		break;
	case TCP_XTEL:
		portname = "XTEL";
		break;
	case TCP_CFINGER:
		portname = "CFINGER";
		break;
	case TCP_POSTGRES:
		portname = "POSTGRES";
		break;
	default:
		portname = NULL;
	}

	return(portname);
}